@@ -163,9 +163,15 @@ SEXP set_factors(SEXP obj, SEXP val, char *nm)
 }
 
 // R interface [for updating the '@ factors' slot of a function *argument* [CARE!]
-SEXP R_set_factors(SEXP obj, SEXP val, SEXP name)
+SEXP R_set_factors(SEXP obj, SEXP val, SEXP name, SEXP warn)
 {
-    return set_factors(obj, val, (char *)CHAR(asChar(name)));
+    Rboolean do_warn = asLogical(warn);
+    if(R_has_slot(obj, Matrix_factorSym))
+	return set_factors(obj, val, (char *)CHAR(asChar(name)));
+    else {
+	if(do_warn) warning(_("Matrix object has no 'factors' slot"));
+	return val;
+    }
 }
 
 #if 0 				/* unused */
@@ -181,9 +187,8 @@ SEXP CMatrix_set_Dim(SEXP x, int nrow)
 #endif	/* unused */
 
 /* Fill in the "trivial remainder" in  n*m  array ;
- *  typically the 'x' slot of a "dtrMatrix" :
- * But should be usable for double/logical/int/complex : */
-
+ *  typically the 'x' slot of a "dtrMatrix", such that
+ *  it should be usable for double/logical/int/complex : */
 #define MAKE_TRIANGULAR_BODY(_TO_, _FROM_, _ZERO_, _ONE_)	\
 {								\
     int i, j, *dims = INTEGER(GET_SLOT(_FROM_, Matrix_DimSym));	\
@@ -275,13 +280,13 @@ SEXP check_scalar_string(SEXP sP, char *vals, char *nm)
 }
 
 /* FIXME? Something like this should be part of the R API ?
- *        But then, R has the more general  compute_identical()
- * in src/main/identical.c: Rboolean compute_identical(SEXP x, SEXP y);
+ *        But then, R has the more general  R_compute_identical()
+ * in src/main/identical.c: Rboolean R_compute_identical(SEXP x, SEXP y);
 */
 Rboolean equal_string_vectors(SEXP s1, SEXP s2)
 {
     Rboolean n1 = isNull(s1), n2 = isNull(s2);
-    if (n1 || n2)
+    if (n1 || n2) // if one is NULL : "equal" if both are
 	return (n1 == n2) ? TRUE : FALSE;
     else if (TYPEOF(s1) != STRSXP || TYPEOF(s2) != STRSXP) {
 	error(_("'s1' and 's2' must be \"character\" vectors"));
@@ -305,11 +310,63 @@ Rboolean equal_string_vectors(SEXP s1, SEXP s2)
 SEXP dense_nonpacked_validate(SEXP obj)
 {
     int *dims = INTEGER(GET_SLOT(obj, Matrix_DimSym));
-    if ((dims[0] * dims[1]) != length(GET_SLOT(obj, Matrix_xSym)))
+    if ((((double) dims[0]) * dims[1]) != XLENGTH(GET_SLOT(obj, Matrix_xSym)))
 	return mkString(_("length of x slot != prod(Dim)"));
     return ScalarLogical(1);
 }
 
+SEXP dim_validate(SEXP Dim, const char* name) {
+    if (length(Dim) != 2)
+	return mkString(_("Dim slot must have length 2"));
+    /* if (TYPEOF(Dim) != INTSXP && TYPEOF(Dim) != REALSXP) */
+    /* 	return mkString(_("Dim slot is not numeric")); */
+    if (TYPEOF(Dim) != INTSXP)
+	// TODO?: coerce REALSXP to INTSXP in the "double" case ???
+    	return mkString(_("Dim slot is not integer"));
+    int
+	m = INTEGER(Dim)[0],
+	n = INTEGER(Dim)[1];
+    if (m < 0 || n < 0)
+	return mkString(dngettext(name,
+				  "Negative value in Dim",
+				  "Negative values in Dim",
+				  (m*n > 0) ? 2 : 1));
+    return ScalarLogical(1);
+}
+// to be called from R :
+SEXP Dim_validate(SEXP obj, SEXP name) {
+    return dim_validate(GET_SLOT(obj, Matrix_DimSym),
+			CHAR(STRING_ELT(name, 0)));
+}
+
+// obj must have @Dim and @Dimnames. Assume 'Dim' is already checked.
+SEXP dimNames_validate(SEXP obj)
+{
+    int *dims = INTEGER(GET_SLOT(obj, Matrix_DimSym));
+    SEXP dmNms = GET_SLOT(obj, Matrix_DimNamesSym);
+    if(!isNewList(dmNms))
+	return mkString(_("Dimnames slot is not a list"));
+    if(length(dmNms) != 2)
+	return mkString(_("Dimnames slot is a list, but not of length 2"));
+    char buf[99];
+    for(int j=0; j < 2; j++) { // x@Dimnames[j] must be NULL or character(length(x@Dim[j]))
+	if(!isNull(VECTOR_ELT(dmNms, j))) {
+	    if(TYPEOF(VECTOR_ELT(dmNms, j)) != STRSXP) {
+		sprintf(buf, _("Dimnames[%d] is not a character vector"), j+1);
+		return mkString(buf);
+	    }
+	    if(LENGTH(VECTOR_ELT(dmNms, j)) != 0 && // character(0) allowed here
+	       LENGTH(VECTOR_ELT(dmNms, j)) != dims[j]) {
+		sprintf(buf, _("length(Dimnames[%d]) differs from Dim[%d] which is %d"),
+			j+1, j+1, dims[j]);
+		return mkString(buf);
+	    }
+	}
+    }
+    return ScalarLogical(1);
+}
+
+
 
 #define PACKED_TO_FULL(TYPE)						\
 TYPE *packed_to_full_ ## TYPE(TYPE *dest, const TYPE *src,		\
@@ -408,7 +465,7 @@ SEXP d_packed_setDiag(double *diag, int l_d, SEXP x, int n)
 	r_x = GET_SLOT(ret, Matrix_xSym);		\
     Rboolean d_full = (l_d == n);			\
     if (!d_full && l_d != 1)				\
-	error("replacement diagonal has wrong length")
+	error(_("replacement diagonal has wrong length"))
 
 #define END_packed_setDiag						\
     int j, pos = 0;							\
@@ -577,56 +634,39 @@ install_diagonal_int(int *dest, SEXP A)
 }
 
 
-/** Duplicate a [dln]denseMatrix _or_ a numeric matrix or even vector as
- *  a [dln]geMatrix.
+/** @brief Duplicate a [dln]denseMatrix _or_ a numeric matrix or even a vector
+ *  as a [dln]geMatrix.
+ *
  *  This is for the many "*_matrix_{prod,crossprod,tcrossprod, etc.}"
  *  functions that work with both classed and unclassed matrices.
  *
- * @param A	  either a denseMatrix object or a matrix object
+ *  Used generally for Generalized -- "geMatrix" -- dispatch where needed.
+ *
+ * @param A either a denseMatrix, a diagonalMatrix or a traditional matrix object
+ *
  */
-/* NOTA BENE: If you enlarge this list, do change '14' and '6' below !
- * ---------  ddiMatrix & ldiMatrix  are no longer ddense or ldense on the R level,
- *            ---         ---        but are still dealt with here.
- */
-
-#define ddense_CLASSES							\
-		    "dgeMatrix", "dtrMatrix",				\
-		    "dsyMatrix", "dpoMatrix", "ddiMatrix",		\
-		    "dtpMatrix", "dspMatrix", "dppMatrix",		\
-		    /* sub classes of those above:*/			\
-		    /* dtr */ "Cholesky", "LDL", "BunchKaufman",	\
-		    /* dtp */ "pCholesky", "pBunchKaufman",		\
-		    /* dpo */ "corMatrix"
-
-#define ldense_CLASSES					\
-		    "lgeMatrix", "ltrMatrix",		\
-		    "lsyMatrix", "ldiMatrix",		\
-		    "ltpMatrix", "lspMatrix"
-
-#define ndense_CLASSES					\
-		    "ngeMatrix", "ntrMatrix",		\
-		    "nsyMatrix",			\
-		    "ntpMatrix", "nspMatrix"
-
-/* Generalized -- "geMatrix" -- dispatch where needed : */
 SEXP dup_mMatrix_as_geMatrix(SEXP A)
 {
-    SEXP ans, ad = R_NilValue, an = R_NilValue;	/* -Wall */
+ /* NOTA BENE: If you enlarge this list, do change '14' and '6' below !
+  * ---------  ddiMatrix & ldiMatrix  are no longer ddense or ldense on the R level,
+  *            ---         ---        but are still dealt with here: */
     static const char *valid[] = {
-	"_NOT_A_CLASS_",/* *_CLASSES defined in ./Mutils.h */
-	ddense_CLASSES, /* 14 */
-	ldense_CLASSES, /* 6  */
-	ndense_CLASSES, /* 5  */
+	"_NOT_A_CLASS_",// *_CLASSES defined in ./Mutils.h  :
+	MATRIX_VALID_ddense, /* 14 */
+	MATRIX_VALID_ldense, /* 6  */
+	MATRIX_VALID_ndense, /* 5  */
 	""};
-    int sz, ctype = Matrix_check_class_etc(A, valid),
+    SEXP ans, ad = R_NilValue, an = R_NilValue;	/* -Wall */
+    int sz, ctype = R_check_class_etc(A, valid),
 	nprot = 1;
-    enum dense_enum { ddense, ldense, ndense } M_type = ddense /* -Wall */;
+    enum dense_enum M_type = ddense /* -Wall */;
 
-    if (ctype > 0) { /* a [nld]denseMatrix object */
-	ad = GET_SLOT(A, Matrix_DimSym);
-	an = GET_SLOT(A, Matrix_DimNamesSym);
+    if (ctype > 0) { /* a [nld]denseMatrix or [dl]diMatrix object */
 	M_type = (ctype <= 14) ? ddense :
 	    ((ctype <= 14+6) ? ldense : ndense);
+	/// TODO: Return 'A' unduplicated if ctype indicates "?geMatrix"' -- and *new argument* 'check' is TRUE
+	ad = GET_SLOT(A, Matrix_DimSym);
+	an = GET_SLOT(A, Matrix_DimNamesSym);
     }
     else if (ctype < 0) {	/* not a (recognized) classed matrix */
 
@@ -643,26 +683,31 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	    error(_("invalid class '%s' to dup_mMatrix_as_geMatrix"),
 		  class_P(A));
 
-#define	DUP_MMATRIX_NON_CLASS						\
+#define	DUP_MMATRIX_NON_CLASS(transpose_if_vec)				\
 	if (isMatrix(A)) {	/* "matrix" */				\
 	    ad = getAttrib(A, R_DimSymbol);				\
 	    an = getAttrib(A, R_DimNamesSymbol);			\
 	} else {/* maybe "numeric" (incl integer,logical) --> (n x 1) */ \
 	    int* dd = INTEGER(ad = PROTECT(allocVector(INTSXP, 2)));	\
 	    nprot++;							\
-	    dd[0] = LENGTH(A);						\
-	    dd[1] = 1;							\
+	    if(transpose_if_vec) {					\
+		dd[0] = 1;						\
+		dd[1] = LENGTH(A);					\
+	    } else {							\
+		dd[0] = LENGTH(A);					\
+		dd[1] = 1;						\
+	    }								\
 	    SEXP nms = getAttrib(A, R_NamesSymbol);			\
 	    if(nms != R_NilValue) {					\
 		an = PROTECT(allocVector(VECSXP, 2));			\
 		nprot++;						\
-	        SET_VECTOR_ELT(an, 0, nms);				\
+	        SET_VECTOR_ELT(an, (transpose_if_vec)? 1 : 0, nms);	\
 		/* not needed: SET_VECTOR_ELT(an, 1, R_NilValue); */    \
 	    } /* else nms = NULL ==> an remains NULL */                 \
  	}								\
 	ctype = 0
 
-	DUP_MMATRIX_NON_CLASS;
+	DUP_MMATRIX_NON_CLASS(FALSE);
     }
 
     ans = PROTECT(NEW_OBJECT(MAKE_CLASS(M_type == ddense ? "dgeMatrix" :
@@ -719,8 +764,7 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	    break;								\
 	}  /* switch(ctype) */
 
-	DUP_MMATRIX_ddense_CASES
-
+	DUP_MMATRIX_ddense_CASES;
     }
     else { /* M_type == ldense || M_type = ndense  */
 	/* ldense -> lge */
@@ -747,18 +791,18 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	    make_i_matrix_symmetric(ansx, A);
 	    break;
 	case 4+14:			/* ldiMatrix */
-	case 4+14+6:			/* ndiMatrix */
+	    // case 4+14+6:      /* ndiMatrix _DOES NOT EXIST_ */
 	    install_diagonal_int(ansx, A);
 	    break;
 	case 5+14:			/* ltpMatrix */
-	case 5+14+6:			/* ntpMatrix */
+	case 4+14+6:			/* ntpMatrix */
 	    packed_to_full_int(ansx, LOGICAL(GET_SLOT(A, Matrix_xSym)),
 			       INTEGER(ad)[0],
 			       *uplo_P(A) == 'U' ? UPP : LOW);
 	    make_i_matrix_triangular(ansx, A);
 	    break;
 	case 6+14:			/* lspMatrix */
-	case 6+14+6:			/* nspMatrix */
+	case 5+14+6:			/* nspMatrix */
 	    packed_to_full_int(ansx, LOGICAL(GET_SLOT(A, Matrix_xSym)),
 			       INTEGER(ad)[0],
 			       *uplo_P(A) == 'U' ? UPP : LOW);
@@ -775,12 +819,12 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
     return ans;
 }
 
-SEXP dup_mMatrix_as_dgeMatrix(SEXP A)
+SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec)
 {
     SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
 	ad = R_NilValue , an = R_NilValue;	/* -Wall */
-    static const char *valid[] = {"_NOT_A_CLASS_", ddense_CLASSES, ""};
-    int ctype = Matrix_check_class_etc(A, valid),
+    static const char *valid[] = {"_NOT_A_CLASS_", MATRIX_VALID_ddense, ""};
+    int ctype = R_check_class_etc(A, valid),
 	nprot = 1, sz;
     double *ansx;
 
@@ -789,26 +833,26 @@ SEXP dup_mMatrix_as_dgeMatrix(SEXP A)
 	an = GET_SLOT(A, Matrix_DimNamesSym);
     }
     else if (ctype < 0) {	/* not a (recognized) classed matrix */
-
-	DUP_MMATRIX_NON_CLASS;
-
-	if (isInteger(A) || isLogical(A)) {
-	    A = PROTECT(coerceVector(A, REALSXP));
-	    nprot++;
+	if (!isReal(A)) {
+	    if (isInteger(A) || isLogical(A)) {
+		A = PROTECT(coerceVector(A, REALSXP));
+		nprot++;
+	    } else
+		error(_("invalid class '%s' to dup_mMatrix_as_dgeMatrix"),
+		      class_P(A));
 	}
-	if (!isReal(A))
-	    error(_("invalid class '%s' to dup_mMatrix_as_dgeMatrix"),
-		  class_P(A));
+	DUP_MMATRIX_NON_CLASS(tr_if_vec);
     }
 
     DUP_MMATRIX_SET_1;
-
-    DUP_MMATRIX_ddense_CASES
-
+    DUP_MMATRIX_ddense_CASES;
     UNPROTECT(nprot);
     return ans;
 }
 
+SEXP dup_mMatrix_as_dgeMatrix(SEXP A) {
+    return dup_mMatrix_as_dgeMatrix2(A, FALSE);
+}
 
 SEXP new_dgeMatrix(int nrow, int ncol)
 {
@@ -834,41 +878,47 @@ SEXP new_dgeMatrix(int nrow, int ncol)
  *
  * @return encoded index; integer if prod(dim) is small; double otherwise
  */
-SEXP m_encodeInd(SEXP ij, SEXP di, SEXP chk_bnds)
+SEXP m_encodeInd(SEXP ij, SEXP di, SEXP orig_1, SEXP chk_bnds)
 {
     SEXP ans;
-    int *ij_di = NULL, n, *Di = INTEGER(di), *IJ, *j_;
-    Rboolean check_bounds = asLogical(chk_bnds);
+    int *ij_di = NULL, n, nprot=1;
+    Rboolean check_bounds = asLogical(chk_bnds), one_ind = asLogical(orig_1);
 
-    ij = PROTECT(coerceVector(ij, INTSXP));
+    if(TYPEOF(di) != INTSXP) {di = PROTECT(coerceVector(di, INTSXP)); nprot++; }
+    if(TYPEOF(ij) != INTSXP) {ij = PROTECT(coerceVector(ij, INTSXP)); nprot++; }
     if(!isMatrix(ij) ||
        (ij_di = INTEGER(getAttrib(ij, R_DimSymbol)))[1] != 2)
 	error(_("Argument ij must be 2-column integer matrix"));
     n = ij_di[0];
-    IJ = INTEGER(ij);
-    j_ = IJ+n;/* pointer offset! */
+    int *Di = INTEGER(di), *IJ = INTEGER(ij),
+	*j_ = IJ+n;/* pointer offset! */
 
     if((Di[0] * (double) Di[1]) >= 1 + (double)INT_MAX) { /* need double */
 	ans = PROTECT(allocVector(REALSXP, n));
 	double *ii = REAL(ans), nr = (double) Di[0];
-#define do_ii_FILL(_i_, _j_)					\
+#define do_ii_FILL(_i_, _j_)						\
 	int i;								\
 	if(check_bounds) {						\
 	    for(i=0; i < n; i++) {					\
 		if(_i_[i] == NA_INTEGER || _j_[i] == NA_INTEGER)	\
 		    ii[i] = NA_INTEGER;					\
 		else {							\
-		    if(_i_[i] < 0 || _i_[i] >= Di[0])			\
+		    register int i_i, j_i;				\
+	            if(one_ind) { i_i = _i_[i]-1; j_i = _j_[i]-1; }	\
+	            else        { i_i = _i_[i]  ; j_i = _j_[i]  ; }	\
+		    if(i_i < 0 || i_i >= Di[0])				\
 			error(_("subscript 'i' out of bounds in M[ij]")); \
-		    if(_j_[i] < 0 || _j_[i] >= Di[1])			\
+		    if(j_i < 0 || j_i >= Di[1])				\
 			error(_("subscript 'j' out of bounds in M[ij]")); \
-		    ii[i] = _i_[i] + _j_[i] * nr;			\
+		    ii[i] = i_i + j_i * nr;				\
 		}							\
 	    }								\
 	} else {							\
 	    for(i=0; i < n; i++)					\
 		ii[i] = (_i_[i] == NA_INTEGER || _j_[i] == NA_INTEGER)	\
-		    ? NA_INTEGER : _i_[i] + _j_[i] * nr;		\
+		    ? NA_INTEGER					\
+ 	            : (one_ind ? ((_i_[i]-1) + (_j_[i]-1)*nr)		\
+	                       :   _i_[i]    +  _j_[i]   *nr);		\
 	}
 
 	do_ii_FILL(IJ, j_);
@@ -878,7 +928,7 @@ SEXP m_encodeInd(SEXP ij, SEXP di, SEXP chk_bnds)
 
 	do_ii_FILL(IJ, j_);
     }
-    UNPROTECT(2);
+    UNPROTECT(nprot);
     return ans;
 }
 
@@ -887,22 +937,24 @@ SEXP m_encodeInd(SEXP ij, SEXP di, SEXP chk_bnds)
  *
  * @param i: integer vector
  * @param j: integer vector of same length as 'i'
+ * @param orig_1: logical: if TRUE, "1-origin" otherwise "0-origin"
  * @param di: dim(.), i.e. length 2 integer vector
  * @param chk_bnds: logical indicating  0 <= ij[,k] < di[k]  need to be checked.
  *
  * @return encoded index; integer if prod(dim) is small; double otherwise
  */
-SEXP m_encodeInd2(SEXP i, SEXP j, SEXP di, SEXP chk_bnds)
+SEXP m_encodeInd2(SEXP i, SEXP j, SEXP di, SEXP orig_1, SEXP chk_bnds)
 {
     SEXP ans;
-    int n = LENGTH(i);
-    int *Di = INTEGER(di), *i_, *j_;
-    Rboolean check_bounds = asLogical(chk_bnds);
+    int n = LENGTH(i), nprot = 1;
+    Rboolean check_bounds = asLogical(chk_bnds), one_ind = asLogical(orig_1);
 
-    if(LENGTH(j) != n || !isInteger(i) || !isInteger(j))
+    if(TYPEOF(di)!= INTSXP) {di = PROTECT(coerceVector(di,INTSXP)); nprot++; }
+    if(TYPEOF(i) != INTSXP) { i = PROTECT(coerceVector(i, INTSXP)); nprot++; }
+    if(TYPEOF(j) != INTSXP) { j = PROTECT(coerceVector(j, INTSXP)); nprot++; }
+    if(LENGTH(j) != n)
 	error(_("i and j must be integer vectors of the same length"));
-    i_ = INTEGER(i);
-    j_ = INTEGER(j);
+    int *Di = INTEGER(di), *i_ = INTEGER(i), *j_ = INTEGER(j);
 
     if((Di[0] * (double) Di[1]) >= 1 + (double)INT_MAX) { /* need double */
 	ans = PROTECT(allocVector(REALSXP, n));
@@ -915,51 +967,11 @@ SEXP m_encodeInd2(SEXP i, SEXP j, SEXP di, SEXP chk_bnds)
 
 	do_ii_FILL(i_, j_);
     }
-    UNPROTECT(1);
+    UNPROTECT(nprot);
     return ans;
 }
 #undef do_ii_FILL
 
-// fails, as R_SVN_REVISION is a string #if R_SVN_REVISION < 60943
-//__ no "if" so it *runs* in older R even if installed in R >= 2.15.2:
-//__ #if R_VERSION < R_Version(2, 15, 2)
-// it is *hidden* in earlier versions of R
-void copyListMatrix(SEXP s, SEXP t, Rboolean byrow)
-{
-    SEXP pt, tmp;
-    int i, j, nr, nc;
-    R_xlen_t ns;
-
-    nr = nrows(s);
-    nc = ncols(s);
-    ns = ((R_xlen_t) nr) * nc;
-    pt = t;
-    if(byrow) {
-	R_xlen_t NR = nr;
-	PROTECT(tmp = allocVector(STRSXP, ns));
-	for (i = 0; i < nr; i++)
-	    for (j = 0; j < nc; j++) {
-		SET_STRING_ELT(tmp, i + j * NR, duplicate(CAR(pt)));
-		pt = CDR(pt);
-		if(pt == R_NilValue) pt = t;
-	    }
-	for (i = 0; i < ns; i++) {
-	    SETCAR(s, STRING_ELT(tmp, i++));
-	    s = CDR(s);
-	}
-	UNPROTECT(1);
-    }
-    else {
-	for (i = 0; i < ns; i++) {
-	    SETCAR(s, duplicate(CAR(pt)));
-	    s = CDR(s);
-	    pt = CDR(pt);
-	    if(pt == R_NilValue) pt = t;
-	}
-    }
-}
-//__ #endif
-
 // Almost "Cut n Paste" from ...R../src/main/array.c  do_matrix() :
 // used in ../R/Matrix.R as
 //
@@ -1081,6 +1093,7 @@ SEXP Mmatrix(SEXP args)
 	    }
 	    break;
 	case RAWSXP:
+	    // FIXME:  N may overflow size_t !!
 	    memset(RAW(ans), 0, N);
 	    break;
 	default:
@@ -1094,6 +1107,71 @@ SEXP Mmatrix(SEXP args)
     return ans;
 }
 
+/**
+ * From the two 'x' slots of two dense matrices a and b,
+ * compute the 'x' slot of rbind(a, b)
+ *
+ * Currently, an auxiliary only for setMethod rbind2(<denseMatrix>, <denseMatrix>)
+ * in ../R/bind2.R
+ *
+ * @param a
+ * @param b
+ *
+ * @return
+ */SEXP R_rbind2_vector(SEXP a, SEXP b) {
+    int *d_a = INTEGER(GET_SLOT(a, Matrix_DimSym)),
+	*d_b = INTEGER(GET_SLOT(b, Matrix_DimSym)),
+	n1 = d_a[0], m = d_a[1],
+	n2 = d_b[0],
+	nprot = 1;
+    SEXP ans,
+	a_x = GET_SLOT(a, Matrix_xSym),
+	b_x = GET_SLOT(b, Matrix_xSym);
+    if(d_b[1] != m)
+	error(_("the number of columns differ in R_rbind2_vector: %d != %d"),
+	      m, d_b[1]);
+    // Care: can have "ddenseMatrix" "ldenseMatrix" or "ndenseMatrix"
+    if(TYPEOF(a_x) != TYPEOF(b_x)) { // choose the "common type"
+	// Now know: either LGLSXP or REALSXP. FIXME for iMatrix, zMatrix,..
+	if(TYPEOF(a_x) != REALSXP)
+	    a_x = PROTECT(duplicate(coerceVector(a_x, REALSXP)));
+	else if(TYPEOF(b_x) != REALSXP)
+	    b_x = PROTECT(duplicate(coerceVector(b_x, REALSXP)));
+	nprot++;
+    }
+
+    ans = PROTECT(allocVector(TYPEOF(a_x), m * (n1 + n2)));
+    int i, ii = 0;
+    switch(TYPEOF(a_x)) {
+    case LGLSXP: {
+	int
+	    *r = LOGICAL(ans),
+	    *ax= LOGICAL(a_x),
+	    *bx= LOGICAL(b_x);
+
+#define COPY_a_AND_b_j						\
+/*  FIXME faster: use Memcpy() : */				\
+	for(int j=0; j < m; j++) {				\
+	    for(i=0; i < n1; i++) r[ii++] = ax[j*n1 + i];	\
+	    for(i=0; i < n2; i++) r[ii++] = bx[j*n2 + i];	\
+	}
+
+	COPY_a_AND_b_j;
+    }
+    case REALSXP: {
+	double
+	    *r = REAL(ans),
+	    *ax= REAL(a_x),
+	    *bx= REAL(b_x);
+
+	COPY_a_AND_b_j;
+    }
+    } // switch
+
+    UNPROTECT(nprot);
+    return ans;
+}
+
 #define TRUE_  ScalarLogical(1)
 #define FALSE_ ScalarLogical(0)
 
@@ -1101,6 +1179,11 @@ SEXP Mmatrix(SEXP args)
 // all0     <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
 // allFalse <- function(x) !any(x) && !any(is.na(x)) ## ~= all0
 SEXP R_all0(SEXP x) {
+    if (!isVectorAtomic(x)) {
+	if(length(x) == 0) return TRUE_;
+	// Typically S4.  TODO: Call the R code above, instead!
+	error(_("Argument must be numeric-like atomic vector"));
+    }
     R_xlen_t i, n = XLENGTH(x);
     if(n == 0) return TRUE_;
 
@@ -1138,6 +1221,11 @@ SEXP R_all0(SEXP x) {
 // any0 <- function(x) isTRUE(any(x == 0)) ## ~= anyFalse
 // anyFalse <- function(x) isTRUE(any(!x)) ## ~= any0
 SEXP R_any0(SEXP x) {
+    if (!isVectorAtomic(x)) {
+	if(length(x) == 0) return FALSE_;
+	// Typically S4.  TODO: Call the R code above, instead!
+	error(_("Argument must be numeric-like atomic vector"));
+    }
     R_xlen_t i, n = XLENGTH(x);
     if(n == 0) return FALSE_;
 
@@ -1169,3 +1257,83 @@ SEXP R_any0(SEXP x) {
 
 #undef TRUE_
 #undef FALSE_
+
+/* FIXME: Compare and synchronize with MK_SYMMETRIC_DIMNAMES.. in ./dense.c
+ * -----  which *also* considers  names(dimnames(.)) !!
+*/
+
+/**
+ * Produce symmetric 'Dimnames' from possibly asymmetric ones.
+ *
+ * @param dn  list of length 2; typically 'Dimnames' slot of "symmetricMatrix"
+ */
+SEXP symmetric_DimNames(SEXP dn) {
+    Rboolean do_nm = FALSE;
+#define NON_TRIVIAL_DN					\
+  !isNull(VECTOR_ELT(dn, 0)) ||				\
+  !isNull(VECTOR_ELT(dn, 1)) ||				\
+ (do_nm = !isNull(getAttrib(dn, R_NamesSymbol)))
+
+#define SYMM_DIMNAMES							\
+	/* Fixup dimnames to be symmetric <==>				\
+	   symmetricDimnames() in ../R/symmetricMatrix.R */		\
+	PROTECT(dn = duplicate(dn));					\
+	if (isNull(VECTOR_ELT(dn,0)))					\
+	    SET_VECTOR_ELT(dn, 0, VECTOR_ELT(dn, 1));			\
+	if (isNull(VECTOR_ELT(dn,1)))					\
+	    SET_VECTOR_ELT(dn, 1, VECTOR_ELT(dn, 0));			\
+	if(do_nm) { /* names(dimnames(.)): */				\
+	    SEXP nms_dn = getAttrib(dn, R_NamesSymbol);			\
+	    if(!R_compute_identical(STRING_ELT(nms_dn, 0),		\
+				    STRING_ELT(nms_dn, 1), 16)) {	\
+		PROTECT(nms_dn);					\
+		int J = LENGTH(STRING_ELT(nms_dn, 0)) == 0; /* 0/1 */   \
+		SET_STRING_ELT(nms_dn, !J, STRING_ELT(nms_dn, J));	\
+		setAttrib(dn, R_NamesSymbol, nms_dn);			\
+		UNPROTECT(1);						\
+            }								\
+	}								\
+	UNPROTECT(1)
+
+    // Be fast (do nothing!) for the case where dimnames = list(NULL,NULL) :
+    if (NON_TRIVIAL_DN) {
+	SYMM_DIMNAMES;
+    }
+    return dn;
+}
+
+/**
+ * Even if the Dimnames slot is list(NULL, <names>) etc, return
+ * __symmetric__ dimnames: Get . @Dimnames and modify when needed.
+ *
+ * Called e.g., from symmetricDimnames() in ../R/symmetricMatrix.R
+ *
+ * @param from a symmetricMatrix
+ *
+ * @return symmetric dimnames: length-2 list twice the same, NULL or
+ * character vector (of correct length)
+ */
+SEXP R_symmetric_Dimnames(SEXP x) {
+    return symmetric_DimNames(GET_SLOT(x, Matrix_DimNamesSym));
+}
+
+
+/**
+ * Set 'Dimnames' slot of 'dest' from the one of 'src' when
+ * 'src' is a "symmetricMatrix" with possibly asymmetric dimnames,
+ * and 'dest' must contain corresponding symmetric dimnames.
+ *
+ * @param dest Matrix, typically *not* symmetric
+ * @param src  symmetricMatrix
+ */
+void SET_DimNames_symm(SEXP dest, SEXP src) {
+    SEXP dn = GET_SLOT(src, Matrix_DimNamesSym);
+    Rboolean do_nm = FALSE;
+    // Be fast (do nothing!) for the case where dimnames = list(NULL,NULL) :
+    if (NON_TRIVIAL_DN) {
+	SYMM_DIMNAMES;
+	SET_SLOT(dest, Matrix_DimNamesSym, dn);
+    }
+    return;
+}
+
